function G=IGVKernel(u,v,eta,sigma,order,fig);

%% (c) 2004 Marco Cuturi , marco.cuturi@ensmp.fr
%% Distributed under GPL licence


%%%%%%% Implements Kernelized version of the IGV on two sets of pixels

%%%%%%% u : image as a line vector of  [L, d, (pixels), (0)] where L is a
%%%%%%% label, d is the number of non-zero pixels, the remaining space
%%%%%%% beging filled with zeros.
%%%%%%% v : same
%%%%%%% eta : regularization
%%%%%%% sigma : Gaussian kernel bandwith.
%%%%%%% if 'order' is a positive integer, the script displays the order^th
%%%%%%% eigenfunction on figure 'fig'. Otherwise nothing is shown.


D=[1/(u(2))*ones(1,u(2)),1/(v(2))*ones(1,v(2))]; % Weight Matrix
D=D/2;                                           % Normalization
w=union(u(3:2+u(2)),v(3:2+v(2)));                % Looking for a base of the support of mu_1 + mu_2
us=zeros(size(w));                               % " 
[dummy,n]=size(w);                               % "
V=[u(3:2+u(2)),v(3:2+v(2))];                     % All enumerated points.

i=zeros(1,n);                               
j=zeros(1,n);

for i=1:n
    for j=1:(u(2)+v(2))
        if (w(i)==V(j))
           us(i)=us(i)+D(j);                     % Plain Computation of their weights
       end
   end
end

R=diag(us);
K=zeros(n,n);

S=2*sigma^2;

if (sigma>0)                                    % Computing Gram Matrix (linear of RBF)
    for i=1:n
        for j=i:n
            K(i,j)=RBF(w(i),w(j),S);
            K(j,i)=K(i,j);
        end
    end
else
    for i=1:n
        for j=i:n
            K(i,j)=dotp(w(i),w(j));
            K(j,i)=K(i,j);
        end
    end
end
%%%%%%% Computing Centered Gram Matrix %%%%%
C=(eye(n)-R*ones(n,n));
B=C'*K*C;
%%%%%%% Inverse of Regularized Generalized Variance %%%%%
G=log(det(1/eta*B*R+eye(n,n)));

%%%%%%% Displaying order^th eigenfunction %%%%%%%%%%%%%%%%%%
if (order>0) 
    B=(transpose(B)+B)/2+0.000000001*eye(n); % Some Minor Regularization
    A=B*R*B;
    [V,T]=eig(A,B);
    figure(fig);
    imdisplay(u,v,w,V(:,order),sigma,us);
end;